@@ -3,7 +3,7 @@
 
 struct PyGeventAresChannelObject;
 
-/* "gevent/ares.pyx":236
+/* "gevent/ares.pyx":235
  * 
  * 
  * cdef public class channel [object PyGeventAresChannelObject, type PyGeventAresChannel_Type]:             # <<<<<<<<<<<<<<
@@ -29,6 +29,10 @@ struct PyGeventAresChannelObject {
   #endif
 #endif
 
+#ifndef DL_IMPORT
+  #define DL_IMPORT(_T) _T
+#endif
+
 __PYX_EXTERN_C DL_IMPORT(PyTypeObject) PyGeventAresChannel_Type;
 
 #endif /* !__PYX_HAVE_API__gevent__ares */
