@@ -9,8 +9,10 @@ MAPPING = {'gevent.local': '_threading_local',
            'gevent.socket': 'socket',
            'gevent.select': 'select',
            'gevent.ssl': 'ssl',
-           'gevent.thread': 'thread'}
-
+           'gevent.thread': 'thread',
+           'gevent.subprocess': 'subprocess',
+           'gevent.os': 'os',
+           'gevent.threading': 'threading'}
 
 
 class ANY(object):
@@ -20,18 +22,25 @@ class ANY(object):
 ANY = ANY()
 
 NOT_IMPLEMENTED = {
-    'socket': ['CAPI', 'gethostbyaddr', 'gethostbyname_ex', 'getnameinfo'],
+    'socket': ['CAPI'],
     'thread': ['allocate', 'exit_thread', 'interrupt_main', 'start_new'],
-    'select': ANY}
+    'select': ANY,
+    'os': ANY,
+    'threading': ANY}
 
 COULD_BE_MISSING = {
     'socket': ['create_connection', 'RAND_add', 'RAND_egd', 'RAND_status']}
 
+NO_ALL = ['gevent.threading', 'gevent._util', 'gevent._socketcommon']
+
 
 class Test(unittest.TestCase):
 
     def check_all(self):
         "Check that __all__ is present and does not contain invalid entries"
+        if not hasattr(self.module, '__all__'):
+            assert self.modname in NO_ALL
+            return
         names = {}
         exec ("from %s import *" % self.modname) in names
         names.pop('__builtins__', None)
@@ -84,7 +93,7 @@ class Test(unittest.TestCase):
         for name in self.__imports__:
             item = getattr(self.module, name)
             stdlib_item = getattr(self.stdlib_module, name)
-            assert item is stdlib_item, (item, stdlib_item)
+            assert item is stdlib_item, (name, item, stdlib_item)
 
     def check_extensions_actually_extend(self):
         """Check that the module actually defines new entries in __extensions__"""
@@ -95,7 +104,7 @@ class Test(unittest.TestCase):
         """Check that __all__ (or dir()) of the corresponsing stdlib is a subset of __all__ of this module"""
         missed = []
         for name in self.stdlib_all:
-            if name not in self.module.__all__:
+            if name not in getattr(self.module, '__all__', []):
                 missed.append(name)
 
         # handle stuff like ssl.socket and ssl.socket_error which have no reason to be in gevent.ssl.__all__
@@ -129,13 +138,14 @@ are missing from %r:
             raise AssertionError(msg)
 
     def _test(self, modname):
+        if modname.endswith('2') or modname.endswith("279"):
+            return
+        if modname.endswith('3'):
+            return
         self.modname = modname
         exec "import %s" % modname in {}
         self.module = sys.modules[modname]
 
-        if modname == 'gevent.six':
-            return
-
         self.check_all()
 
         self.__implements__ = getattr(self.module, '__implements__', None)
@@ -153,13 +163,13 @@ are missing from %r:
 
         self.check_implements_presence_justified()
 
+        if self.stdlib_module is None:
+            return
+
         # use __all__ as __implements__
         if self.__implements__ is None:
             self.__implements__ = sorted(self.module.__all__)
 
-        if self.stdlib_module is None:
-            return
-
         self.set_stdlib_all()
         self.check_implements_subset_of_stdlib_all()
         self.check_implements_actually_implements()
