@@ -3,6 +3,7 @@ from time import time
 import gevent
 from gevent import pool
 from gevent.event import Event
+from gevent.queue import Queue
 import greentest
 import random
 from greentest import ExpectedException
@@ -226,6 +227,12 @@ def sqr_random_sleep(x):
     return x * x
 
 
+def final_sleep():
+    for i in range(3):
+        yield i
+    gevent.sleep(0.2)
+
+
 TIMEOUT1, TIMEOUT2, TIMEOUT3 = 0.082, 0.035, 0.14
 
 
@@ -331,6 +338,30 @@ class TestPool(greentest.TestCase):
             expected = ['1', '2', '10']
         self.assertEqual(result, expected)
 
+    # https://github.com/gevent/gevent/issues/423
+    def test_imap_no_stop(self):
+        q = Queue()
+        q.put(123)
+        gevent.spawn_later(0.1, q.put, StopIteration)
+        result = list(self.pool.imap(lambda _: _, q))
+        self.assertEqual(result, [123])
+
+    def test_imap_unordered_no_stop(self):
+        q = Queue()
+        q.put(1234)
+        gevent.spawn_later(0.1, q.put, StopIteration)
+        result = list(self.pool.imap_unordered(lambda _: _, q))
+        self.assertEqual(result, [1234])
+
+    # same issue, but different test: https://github.com/gevent/gevent/issues/311
+    def test_imap_final_sleep(self):
+        result = list(self.pool.imap(sqr, final_sleep()))
+        self.assertEqual(result, [0, 1, 4])
+
+    def test_imap_unordered_final_sleep(self):
+        result = list(self.pool.imap_unordered(sqr, final_sleep()))
+        self.assertEqual(result, [0, 1, 4])
+
 
 class TestPool2(TestPool):
     size = 2
@@ -416,5 +447,33 @@ class TestErrorInIterator(greentest.TestCase):
         gevent.sleep(0.001)
 
 
+def divide_by(x):
+    return 1.0 / x
+
+
+class TestErrorInHandler(greentest.TestCase):
+    error_fatal = False
+
+    def test_map(self):
+        p = pool.Pool(3)
+        self.assertRaises(ZeroDivisionError, p.map, divide_by, [1, 0, 2])
+
+    def test_imap(self):
+        p = pool.Pool(1)
+        it = p.imap(divide_by, [1, 0, 2])
+        self.assertEqual(it.next(), 1.0)
+        self.assertRaises(ZeroDivisionError, it.next)
+        self.assertEqual(it.next(), 0.5)
+        self.assertRaises(StopIteration, it.next)
+
+    def test_imap_unordered(self):
+        p = pool.Pool(1)
+        it = p.imap_unordered(divide_by, [1, 0, 2])
+        self.assertEqual(it.next(), 1.0)
+        self.assertRaises(ZeroDivisionError, it.next)
+        self.assertEqual(it.next(), 0.5)
+        self.assertRaises(StopIteration, it.next)
+
+
 if __name__ == '__main__':
     greentest.main()
