@@ -34,6 +34,7 @@ except ImportError:
 import greentest
 import gevent
 from gevent import socket
+from gevent import pywsgi
 from gevent.pywsgi import Input
 
 
@@ -137,7 +138,7 @@ class Response(object):
     def assertHeader(self, header, value):
         real_value = self.headers.get(header, False)
         assert real_value == value, \
-               'Unexpected header %r: %r (expected %r)\n%s' % (header, real_value, value, self)
+            'Unexpected header %r: %r (expected %r)\n%s' % (header, real_value, value, self)
 
     def assertBody(self, body):
         assert self.body == body, 'Unexpected body: %r (expected %r)\n%s' % (self.body, body, self)
@@ -213,14 +214,10 @@ socket.socket.makefile = makefile
 
 class TestCase(greentest.TestCase):
 
-    def get_wsgi_module(self):
-        from gevent import pywsgi
-        return pywsgi
-
     validator = staticmethod(validator)
 
     def init_server(self, application):
-        self.server = self.get_wsgi_module().WSGIServer(('127.0.0.1', 0), application)
+        self.server = pywsgi.WSGIServer(('127.0.0.1', 0), application)
 
     def setUp(self):
         application = self.application
@@ -374,6 +371,44 @@ class TestYield(CommonTests):
             yield "not found"
 
 
+if sys.version_info[:2] >= (2, 6):
+
+    class TestBytearray(CommonTests):
+
+        validator = None
+
+        @staticmethod
+        def application(env, start_response):
+            path = env['PATH_INFO']
+            if path == '/':
+                start_response('200 OK', [('Content-Type', 'text/plain')])
+                return [bytearray("hello "), bytearray("world")]
+            else:
+                start_response('404 Not Found', [('Content-Type', 'text/plain')])
+                return [bytearray("not found")]
+
+
+class MultiLineHeader(TestCase):
+    @staticmethod
+    def application(env, start_response):
+        assert "test.submit" in env["CONTENT_TYPE"]
+        start_response('200 OK', [('Content-Type', 'text/plain')])
+        return ["ok"]
+
+    def test_multiline_116(self):
+        """issue #116"""
+        request = '\r\n'.join((
+            'POST / HTTP/1.0',
+            'Host: localhost',
+            'Content-Type: multipart/related; boundary="====XXXX====";',
+            ' type="text/xml";start="test.submit"',
+            'Content-Length: 0',
+            '', ''))
+        fd = self.makefile()
+        fd.write(request)
+        read_http(fd)
+
+
 class TestGetArg(TestCase):
 
     @staticmethod
@@ -396,7 +431,7 @@ class TestGetArg(TestCase):
 
         # send some junk after the actual request
         fd.write('01234567890123456789')
-        read_http(fd, version='1.0', body='a is a, body is a=a')
+        read_http(fd, body='a is a, body is a=a')
         fd.close()
 
 
@@ -427,7 +462,7 @@ class TestChunkedApp(TestCase):
     def test_no_chunked_http_1_0(self):
         fd = self.makefile()
         fd.write('GET / HTTP/1.0\r\nHost: localhost\r\nConnection: close\r\n\r\n')
-        response = read_http(fd, version='1.0')
+        response = read_http(fd)
         self.assertEqual(response.body, self.body())
         self.assertEqual(response.headers.get('Transfer-Encoding'), None)
         content_length = response.headers.get('Content-Length')
@@ -529,7 +564,7 @@ class HttpsTestCase(TestCase):
     keyfile = os.path.join(os.path.dirname(__file__), 'test_server.key')
 
     def init_server(self, application):
-        self.server = self.get_wsgi_module().WSGIServer(('127.0.0.1', 0), application, certfile=self.certfile, keyfile=self.keyfile)
+        self.server = pywsgi.WSGIServer(('127.0.0.1', 0), application, certfile=self.certfile, keyfile=self.keyfile)
 
     def urlopen(self, method='GET', post_body=None, **kwargs):
         import ssl
@@ -544,6 +579,7 @@ class HttpsTestCase(TestCase):
                 kwargs['body'] = post_body
         else:
             fd.write('\r\n')
+        fd.flush()
         return read_http(fd, **kwargs)
 
     def application(self, environ, start_response):
@@ -576,7 +612,11 @@ class TestInternational(TestCase):
 
     def test(self):
         sock = self.connect()
-        sock.sendall('GET /%D0%BF%D1%80%D0%B8%D0%B2%D0%B5%D1%82?%D0%B2%D0%BE%D0%BF%D1%80%D0%BE%D1%81=%D0%BE%D1%82%D0%B2%D0%B5%D1%82 HTTP/1.1\r\nHost: localhost\r\nConnection: close\r\n\r\n')
+        sock.sendall('''GET /%D0%BF%D1%80%D0%B8%D0%B2%D0%B5%D1%82?%D0%B2%D0%BE%D0%BF%D1%80%D0%BE%D1%81=%D0%BE%D1%82%D0%B2%D0%B5%D1%82 HTTP/1.1
+Host: localhost
+Connection: close
+
+'''.replace('\n', '\r\n'))
         read_http(sock.makefile(), reason='PASSED', chunks=False, body='', content_length=0)
 
 
@@ -732,6 +772,69 @@ class TestEmptyYield304(TestCase):
         self.assert_(garbage == "", "got garbage: %r" % garbage)
 
 
+class TestContentLength304(TestCase):
+    validator = None
+
+    def application(self, env, start_response):
+        try:
+            start_response('304 Not modified', [('Content-Length', '100')])
+        except AssertionError, ex:
+            start_response('200 Raised', [])
+            return [str(ex)]
+        else:
+            raise AssertionError('start_response did not fail but it should')
+
+    def test_err(self):
+        fd = self.connect().makefile(bufsize=1)
+        fd.write('GET / HTTP/1.1\r\nHost: localhost\r\nConnection: close\r\n\r\n')
+        body = "Invalid Content-Length for 304 response: '100' (must be absent or zero)"
+        read_http(fd, code=200, reason='Raised', body=body, chunks=False)
+        garbage = fd.read()
+        self.assert_(garbage == "", "got garbage: %r" % garbage)
+
+
+class TestBody304(TestCase):
+    validator = None
+
+    def application(self, env, start_response):
+        start_response('304 Not modified', [])
+        return ['body']
+
+    def test_err(self):
+        fd = self.connect().makefile(bufsize=1)
+        fd.write('GET / HTTP/1.1\r\nHost: localhost\r\nConnection: close\r\n\r\n')
+        try:
+            read_http(fd)
+        except AssertionError, ex:
+            self.assertEqual(str(ex), 'The 304 response must have no body')
+        else:
+            raise AssertionError('AssertionError must be raised')
+
+
+class TestWrite304(TestCase):
+    validator = None
+
+    def application(self, env, start_response):
+        write = start_response('304 Not modified', [])
+        self.error_raised = False
+        try:
+            write('body')
+        except AssertionError:
+            self.error_raised = True
+            raise
+
+    def test_err(self):
+        fd = self.connect().makefile(bufsize=1)
+        fd.write('GET / HTTP/1.1\r\nHost: localhost\r\nConnection: close\r\n\r\n')
+        try:
+            read_http(fd)
+        except AssertionError, ex:
+            self.assertEqual(str(ex), 'The 304 response must have no body')
+        else:
+            raise AssertionError('write() must raise')
+        assert self.error_raised, 'write() must raise'
+
+
 class TestEmptyWrite(TestEmptyYield):
 
     @staticmethod
@@ -755,13 +858,13 @@ class BadRequestTests(TestCase):
         self.content_length = '-100'
         fd = self.connect().makefile(bufsize=1)
         fd.write('GET / HTTP/1.1\r\nHost: localhost\r\nContent-Length: %s\r\n\r\n' % self.content_length)
-        read_http(fd, code=(200, 400), version=None)
+        read_http(fd, code=(200, 400))
 
     def test_illegal_content_length(self):
         self.content_length = 'abc'
         fd = self.connect().makefile(bufsize=1)
         fd.write('GET / HTTP/1.1\r\nHost: localhost\r\nContent-Length: %s\r\n\r\n' % self.content_length)
-        read_http(fd, code=(200, 400), version=None)
+        read_http(fd, code=(200, 400))
 
 
 class ChunkedInputTests(TestCase):
@@ -954,13 +1057,13 @@ class TestLeakInput(TestCase):
         fd = self.connect().makefile(bufsize=1)
         fd.write("GET / HTTP/1.0\r\nConnection: close\r\n\r\n")
         d = fd.read()
-        assert d.startswith("HTTP/1.0 200 OK"), "bad response"
+        assert d.startswith("HTTP/1.1 200 OK"), "bad response: %r" % d
 
     def test_connection_close_leak_frame(self):
         fd = self.connect().makefile(bufsize=1)
         fd.write("GET /leak-frame HTTP/1.0\r\nConnection: close\r\n\r\n")
         d = fd.read()
-        assert d.startswith("HTTP/1.0 200 OK"), "bad response"
+        assert d.startswith("HTTP/1.1 200 OK"), "bad response: %r" % d
         self._leak_environ.pop('_leak')
 
 
@@ -979,12 +1082,31 @@ class TestInvalidEnviron(TestCase):
     def test(self):
         fd = self.makefile()
         fd.write('GET / HTTP/1.0\r\nHost: localhost\r\n\r\n')
-        read_http(fd, version='1.0')
+        read_http(fd)
         fd = self.makefile()
         fd.write('GET / HTTP/1.1\r\nHost: localhost\r\n\r\n')
         read_http(fd)
 
 
+class Handler(pywsgi.WSGIHandler):
+
+    def read_requestline(self):
+        data = self.rfile.read(7)
+        if data[0] == '<':
+            try:
+                data += self.rfile.read(15)
+                if data.lower() == '<policy-file-request/>':
+                    self.socket.sendall('HELLO')
+                else:
+                    self.log_error('Invalid request: %r', data)
+            finally:
+                self.socket.shutdown(socket.SHUT_WR)
+                self.socket.close()
+                self.socket = None
+        else:
+            return data + self.rfile.readline()
+
+
 class TestSubclass1(TestCase):
 
     validator = None
@@ -994,39 +1116,22 @@ class TestSubclass1(TestCase):
         return []
 
     def init_server(self, application):
-        WSGIHandler = self.get_wsgi_module().WSGIHandler
-
-        class Handler(WSGIHandler):
-
-            def read_requestline(self):
-                data = self.rfile.read(7)
-                if data[0] == '<':
-                    try:
-                        data += self.rfile.read(15)
-                        if data.lower() == '<policy-file-request/>':
-                            self.socket.sendall('HELLO')
-                        else:
-                            self.log_error('Invalid request: %r', data)
-                    finally:
-                        self.socket.shutdown(socket.SHUT_WR)
-                        self.socket.close()
-                        self.socket = None
-                else:
-                    return data + self.rfile.readline()
-
-        self.server = self.get_wsgi_module().WSGIServer(('127.0.0.1', 0), application, handler_class=Handler)
+        self.server = pywsgi.WSGIServer(('127.0.0.1', 0), application, handler_class=Handler)
 
     def test(self):
         fd = self.makefile()
         fd.write('<policy-file-request/>\x00')
+        fd.flush()
         self.assertEqual(fd.read(), 'HELLO')
 
         fd = self.makefile()
         fd.write('GET / HTTP/1.1\r\nHost: localhost\r\nConnection: close\r\n\r\n')
+        fd.flush()
         read_http(fd)
 
         fd = self.makefile()
         fd.write('<policy-file-XXXuest/>\x00')
+        fd.flush()
         self.assertEqual(fd.read(), '')
 
 
@@ -1121,6 +1226,20 @@ class TestInputRaw(greentest.BaseTestCase):
         i = self.make_input("2\r\n1", chunked_input=True)
         self.assertRaises(IOError, i.readline)
 
+
+class Test414(TestCase):
+
+    @staticmethod
+    def application(env, start_response):
+        raise AssertionError('should not get there')
+
+    def test(self):
+        fd = self.makefile()
+        longline = 'x' * 20000
+        fd.write('''GET /%s HTTP/1.0\r\nHello: world\r\n\r\n''' % longline)
+        read_http(fd, code=414)
+
+
 del CommonTests
 
 if __name__ == '__main__':
